/*
 * Argus Software
 * Copyright (c) 2000-2007 QoSient, LLC
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/* 
 * $Id: //depot/argus/argus-3.0/clients/include/argus_util.h#37 $
 * $DateTime: 2006/05/25 02:07:01 $
 * $Change: 858 $
 */

#ifndef ArgusUtil_h
#define ArgusUtil_h

#include <argus_os.h>
#include <compat.h>

#include <netinet/in_systm.h>
#include <netinet/ip.h>

#include <pthread.h>

#include <argus_def.h>
#include <argus_out.h>
#include <cons_out.h>

#include <CflowdFlowPdu.h>

typedef void (*proc)(void);
typedef char *(*strproc)(void);

struct ArgusQueueHeader {
   struct ArgusQueueHeader *nxt;
   struct ArgusQueueHeader *prv;
   struct ArgusQueueStruct *queue;
   struct timeval lasttime, logtime;
};

struct ArgusQueueStruct {
   unsigned int count;
   struct ArgusQueueHeader *start, *end;
   struct ArgusQueueHeader **array;
};

#include <argus_parser.h>

#if defined(__OpenBSD__)
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#endif

#include <netinet/ip_icmp.h>
#undef ICMP_MAXTYPE
#define ICMP_MAXTYPE	46

 
struct ArgusFileEntry {
   struct ArgusFileEntry *nxt;
   char *str;
};

 
#define ARGUS_NOLOCK     0
#define ARGUS_LOCK       1

#define ARGUS_RFILE_LIST        1
#define ARGUS_WFILE_LIST        2
#define ARGUS_DEVICE_LIST       3
#define ARGUS_OUTPUT_LIST       4


struct ArgusListObjectStruct {
   struct ArgusListObjectStruct *nxt, *prv;
   union {
      void *obj;
      unsigned int val;
   } list_union;
};

#define list_obj	list_union.obj
#define list_val	list_union.val
 
struct ArgusListRecord {
   struct ArgusListObjectStruct *nxt, *prv;
   struct ArgusRecordHeader argus;
};
 
struct ArgusListStruct {
   struct ArgusListObjectStruct *start;
   struct ArgusListObjectStruct *end;
   pthread_mutex_t lock;
   struct timeval outputTime, reportTime;
   unsigned int count;
};

struct ArgusPrintFieldStruct {
   char *field, *format;
   int length, index;
   void (*print)(struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
   void (*label)(struct ArgusParserStruct *, char *);

};

struct tok {
   int v;                  /* value */ 
   const char *s;          /* string */
};  
  

#define Version1        1
#define Version5        5
#define Version6        6
#define Version7        7
#define Version8        8


struct ArgusRecord *ArgusNetFlowCallRecord (struct ArgusInput *, u_char **);
struct ArgusRecord *ArgusNetFlowDetailInt  (struct ArgusInput *, u_char **);
struct ArgusRecord *ArgusParseCiscoRecord (struct ArgusInput *, u_char **);

#ifdef ARGUS_SASL
#include <sasl/sasl.h>
#endif

#include <sys/stat.h>
#include <stdio.h>

#define ARGUS_DATA_SOURCE		0x01
#define ARGUS_V2_DATA_SOURCE		0x02
#define ARGUS_CISCO_DATA_SOURCE		0x10

#define ipaddr_string(p) ArgusGetName(ArgusParser, (u_char *)(p))

#ifdef ArgusUtil

void ArgusPrintStartDate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintLastDate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcStartDate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcLastDate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstStartDate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstLastDate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintRelativeDate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSourceID (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintFlags (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintMacAddress (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcMacAddress (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstMacAddress (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintProto (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintAddr (struct ArgusParserStruct *, char *, int, void *, int, int, int);
void ArgusPrintSrcNet (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcAddr (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstNet (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstAddr (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintPort (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *, int, unsigned char, unsigned short, int, int);
void ArgusPrintSrcPort (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstPort (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDir (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintPackets (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcPackets (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstPackets (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintAppBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcAppBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstAppBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcIntPkt (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstIntPkt (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintActiveIntPkt (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintActiveSrcIntPkt (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintActiveDstIntPkt (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintIdleIntPkt (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintIdleSrcIntPkt (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintIdleDstIntPkt (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcIntPktMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcIntPktMin (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstIntPktMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstIntPktMin (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintActiveSrcIntPktMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintActiveSrcIntPktMin (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintActiveDstIntPktMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintActiveDstIntPktMin (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintIdleSrcIntPktMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintIdleSrcIntPktMin (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintIdleDstIntPktMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintIdleDstIntPktMin (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcJitter (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstJitter (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintActiveSrcJitter (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintActiveDstJitter (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintIdleJitter (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintIdleSrcJitter (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintIdleDstJitter (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintState (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDeltaDuration (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDeltaStartTime (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDeltaLastTime (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDeltaSrcPkts (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDeltaDstPkts (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDeltaSrcBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDeltaDstBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintPercentDeltaSrcPkts (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintPercentDeltaDstPkts (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintPercentDeltaSrcBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintPercentDeltaDstBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcUserData (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstUserData (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintUserData (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintTCPExtensions (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcLoad (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstLoad (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintLoad (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcLoss (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstLoss (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintLoss (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcPercentLoss (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstPercentLoss (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintPercentLoss (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcRate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstRate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintRate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcTos (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstTos (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcDSByte (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstDSByte (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcIpId (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstIpId (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcTtl (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstTtl (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcVlan (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstVlan (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcVID (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstVID (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcVPRI (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstVPRI (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcMpls (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstMpls (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcWindow (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstWindow (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintJoinDelay (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintLeaveDelay (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintAvgDuration (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintMinDuration (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintMaxDuration (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintStdDeviation (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcRange (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstRange (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSrcDuration (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDstDuration (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintDuration (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintTransactions (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintSequenceNumber (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintBinNumber (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintBins (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintTCPSrcBase (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintTCPDstBase (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintTCPRTT (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintTCPSynAck (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintTCPAckDat (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintTCPSrcMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintTCPDstMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintManStatus (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintICMPStatus (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintIGMPStatus (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintIPStatus (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintInode (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
void ArgusPrintByteOffset (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
 
void ArgusPrintStartDateLabel (struct ArgusParserStruct *, char *);
void ArgusPrintLastDateLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcStartDateLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcLastDateLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstStartDateLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstLastDateLabel (struct ArgusParserStruct *, char *);
void ArgusPrintRelativeDateLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSourceIDLabel (struct ArgusParserStruct *, char *);
void ArgusPrintFlagsLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcMacAddressLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstMacAddressLabel (struct ArgusParserStruct *, char *);
void ArgusPrintMacAddressLabel (struct ArgusParserStruct *, char *);
void ArgusPrintProtoLabel (struct ArgusParserStruct *, char *);
void ArgusPrintAddrLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcNetLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcAddrLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstNetLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstAddrLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcPortLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstPortLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcIpIdLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstIpIdLabel (struct ArgusParserStruct *, char *);
void ArgusPrintIpIdLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcTtlLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstTtlLabel (struct ArgusParserStruct *, char *);
void ArgusPrintTtlLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDirLabel (struct ArgusParserStruct *, char *);
void ArgusPrintPacketsLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcPacketsLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstPacketsLabel (struct ArgusParserStruct *, char *);
void ArgusPrintBytesLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcBytesLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstBytesLabel (struct ArgusParserStruct *, char *);
void ArgusPrintAppBytesLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcAppBytesLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstAppBytesLabel (struct ArgusParserStruct *, char *);
void ArgusPrintIntPktLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcIntPktLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstIntPktLabel (struct ArgusParserStruct *, char *);
void ArgusPrintActiveSrcIntPktLabel (struct ArgusParserStruct *, char *);
void ArgusPrintActiveDstIntPktLabel (struct ArgusParserStruct *, char *);
void ArgusPrintIdleIntPktLabel (struct ArgusParserStruct *, char *);
void ArgusPrintIdleSrcIntPktLabel (struct ArgusParserStruct *, char *);
void ArgusPrintIdleDstIntPktLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcIntPktMaxLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcIntPktMinLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstIntPktMaxLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstIntPktMinLabel (struct ArgusParserStruct *, char *);
void ArgusPrintActiveSrcIntPktMaxLabel (struct ArgusParserStruct *, char *);
void ArgusPrintActiveSrcIntPktMinLabel (struct ArgusParserStruct *, char *);
void ArgusPrintActiveDstIntPktMaxLabel (struct ArgusParserStruct *, char *);
void ArgusPrintActiveDstIntPktMinLabel (struct ArgusParserStruct *, char *);
void ArgusPrintIdleSrcIntPktMaxLabel (struct ArgusParserStruct *, char *);
void ArgusPrintIdleSrcIntPktMinLabel (struct ArgusParserStruct *, char *);
void ArgusPrintIdleDstIntPktMaxLabel (struct ArgusParserStruct *, char *);
void ArgusPrintIdleDstIntPktMinLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcJitterLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstJitterLabel (struct ArgusParserStruct *, char *);
void ArgusPrintActiveSrcJitterLabel (struct ArgusParserStruct *, char *);
void ArgusPrintActiveDstJitterLabel (struct ArgusParserStruct *, char *);
void ArgusPrintIdleSrcJitterLabel (struct ArgusParserStruct *, char *);
void ArgusPrintIdleDstJitterLabel (struct ArgusParserStruct *, char *);
void ArgusPrintStateLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDeltaDurationLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDeltaStartTimeLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDeltaLastTimeLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDeltaSrcPktsLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDeltaDstPktsLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDeltaSrcBytesLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDeltaDstBytesLabel (struct ArgusParserStruct *, char *);
void ArgusPrintPercentDeltaSrcPktsLabel (struct ArgusParserStruct *, char *);
void ArgusPrintPercentDeltaDstPktsLabel (struct ArgusParserStruct *, char *);
void ArgusPrintPercentDeltaSrcBytesLabel (struct ArgusParserStruct *, char *);
void ArgusPrintPercentDeltaDstBytesLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcUserDataLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstUserDataLabel (struct ArgusParserStruct *, char *);
void ArgusPrintUserDataLabel (struct ArgusParserStruct *, char *);
void ArgusPrintTCPExtensionsLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcLoadLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstLoadLabel (struct ArgusParserStruct *, char *);
void ArgusPrintLoadLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcLossLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstLossLabel (struct ArgusParserStruct *, char *);
void ArgusPrintLossLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcPercentLossLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstPercentLossLabel (struct ArgusParserStruct *, char *);
void ArgusPrintPercentLossLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcRateLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstRateLabel (struct ArgusParserStruct *, char *);
void ArgusPrintRateLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcTosLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstTosLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcDSByteLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstDSByteLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcVlanLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstVlanLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcVIDLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstVIDLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcVPRILabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstVPRILabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcMplsLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstMplsLabel (struct ArgusParserStruct *, char *);
void ArgusPrintWindowLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcWindowLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstWindowLabel (struct ArgusParserStruct *, char *);
void ArgusPrintJoinDelayLabel (struct ArgusParserStruct *, char *);
void ArgusPrintLeaveDelayLabel (struct ArgusParserStruct *, char *);
void ArgusPrintAvgDurationLabel (struct ArgusParserStruct *, char *);
void ArgusPrintMinDurationLabel (struct ArgusParserStruct *, char *);
void ArgusPrintMaxDurationLabel (struct ArgusParserStruct *, char *);
void ArgusPrintStdDeviationLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcRangeLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstRangeLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSrcDurationLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDstDurationLabel (struct ArgusParserStruct *, char *);
void ArgusPrintDurationLabel (struct ArgusParserStruct *, char *);
void ArgusPrintTransactionsLabel (struct ArgusParserStruct *, char *);
void ArgusPrintSequenceNumberLabel (struct ArgusParserStruct *, char *);
void ArgusPrintBinNumberLabel (struct ArgusParserStruct *, char *);
void ArgusPrintBinsLabel (struct ArgusParserStruct *, char *);
void ArgusPrintTCPSrcBaseLabel (struct ArgusParserStruct *, char *);
void ArgusPrintTCPDstBaseLabel (struct ArgusParserStruct *, char *);
void ArgusPrintTCPRTTLabel (struct ArgusParserStruct *, char *);
void ArgusPrintTCPSynAckLabel (struct ArgusParserStruct *, char *);
void ArgusPrintTCPAckDatLabel (struct ArgusParserStruct *, char *);
void ArgusPrintTCPSrcMaxLabel (struct ArgusParserStruct *, char *);
void ArgusPrintTCPDstMaxLabel (struct ArgusParserStruct *, char *);
void ArgusPrintInodeLabel (struct ArgusParserStruct *, char *);
void ArgusPrintByteOffsetLabel (struct ArgusParserStruct *, char *);

struct ArgusPrintFieldStruct 
RaPrintAlgorithmTable[MAX_PRINT_ALG_TYPES] = {
#define ARGUSPRINTSTARTDATE		0
   { "stime", "%T.%f", 12 , 1, ArgusPrintStartDate, ArgusPrintStartDateLabel},
#define ARGUSPRINTLASTDATE		1
   { "ltime", "%T.%f", 12 , 1, ArgusPrintLastDate, ArgusPrintLastDateLabel},
#define ARGUSPRINTTRANSACTIONS		2
   { "trans", "", 6 , 1, ArgusPrintTransactions, ArgusPrintTransactionsLabel},
#define ARGUSPRINTDURATION		3
   { "dur", "", 10 , 1, ArgusPrintDuration, ArgusPrintDurationLabel},
#define ARGUSPRINTAVGDURATION		4
   { "avgdur", "", 10 , 1, ArgusPrintAvgDuration, ArgusPrintAvgDurationLabel},
#define ARGUSPRINTMINDURATION		5
   { "mindur", "", 10 , 1, ArgusPrintMinDuration, ArgusPrintMinDurationLabel},
#define ARGUSPRINTMAXDURATION		6
   { "maxdur", "", 10 , 1, ArgusPrintMaxDuration, ArgusPrintMaxDurationLabel},
#define ARGUSPRINTSRCADDR		7
   { "saddr", "", 18 , 1, ArgusPrintSrcAddr, ArgusPrintSrcAddrLabel},
#define ARGUSPRINTDSTADDR		8
   { "daddr", "", 18 , 1, ArgusPrintDstAddr, ArgusPrintDstAddrLabel},
#define ARGUSPRINTPROTO			9
   { "proto", "", 6 , 1, ArgusPrintProto, ArgusPrintProtoLabel},
#define ARGUSPRINTSRCPORT		10
   { "sport", "", 6 , 1, ArgusPrintSrcPort, ArgusPrintSrcPortLabel},
#define ARGUSPRINTDSTPORT		11
   { "dport", "", 6 , 1, ArgusPrintDstPort, ArgusPrintDstPortLabel},
#define ARGUSPRINTSRCTOS		12
   { "stos", "", 5 , 1, ArgusPrintSrcTos, ArgusPrintSrcTosLabel},
#define ARGUSPRINTDSTTOS		13
   { "dtos", "", 5 , 1, ArgusPrintDstTos, ArgusPrintDstTosLabel},
#define ARGUSPRINTSRCDSBYTE		14
   { "sdsb", "", 5 , 1, ArgusPrintSrcDSByte, ArgusPrintSrcDSByteLabel},
#define ARGUSPRINTDSTDSBYTE		15
   { "ddsb", "", 5 , 1, ArgusPrintDstDSByte, ArgusPrintDstDSByteLabel},
#define ARGUSPRINTSRCTTL		16
   { "sttl", "", 4 , 1, ArgusPrintSrcTtl, ArgusPrintSrcTtlLabel},
#define ARGUSPRINTDSTTTL		17
   { "dttl", "", 4 , 1, ArgusPrintDstTtl, ArgusPrintDstTtlLabel},
#define ARGUSPRINTBYTES			18
   { "bytes", "", 10 , 1, ArgusPrintBytes, ArgusPrintBytesLabel},
#define ARGUSPRINTSRCBYTES		19
   { "sbytes", "", 12 , 1, ArgusPrintSrcBytes, ArgusPrintSrcBytesLabel},
#define ARGUSPRINTDSTBYTES		20
   { "dbytes", "", 12 , 1, ArgusPrintDstBytes, ArgusPrintDstBytesLabel},
#define ARGUSPRINTAPPBYTES              21
   { "appbytes", "", 10 , 1, ArgusPrintAppBytes, ArgusPrintAppBytesLabel},
#define ARGUSPRINTSRCAPPBYTES           22
   { "sappbytes", "", 12 , 1, ArgusPrintSrcAppBytes, ArgusPrintSrcAppBytesLabel},
#define ARGUSPRINTDSTAPPBYTES           23
   { "dappbytes", "", 12 , 1, ArgusPrintDstAppBytes, ArgusPrintDstAppBytesLabel},
#define ARGUSPRINTPACKETS		24
   { "pkts", "", 8 , 1, ArgusPrintPackets, ArgusPrintPacketsLabel},
#define ARGUSPRINTSRCPACKETS		25
   { "spkts", "", 8 , 1, ArgusPrintSrcPackets, ArgusPrintSrcPacketsLabel},
#define ARGUSPRINTDSTPACKETS		26
   { "dpkts", "", 8 , 1, ArgusPrintDstPackets, ArgusPrintDstPacketsLabel},
#define ARGUSPRINTLOAD			27
   { "load", "", 8 , 1, ArgusPrintLoad, ArgusPrintLoadLabel},
#define ARGUSPRINTSRCLOAD		28
   { "sload", "", 8 , 1, ArgusPrintSrcLoad, ArgusPrintSrcLoadLabel},
#define ARGUSPRINTDSTLOAD		29
   { "dload", "", 8 , 1, ArgusPrintDstLoad, ArgusPrintDstLoadLabel},
#define ARGUSPRINTLOSS			30
   { "loss", "", 10 , 1, ArgusPrintLoss, ArgusPrintLossLabel},
#define ARGUSPRINTSRCLOSS		31
   { "sloss", "", 10 , 1, ArgusPrintSrcLoss, ArgusPrintSrcLossLabel},
#define ARGUSPRINTDSTLOSS		32
   { "dloss", "", 10 , 1, ArgusPrintDstLoss, ArgusPrintDstLossLabel},
#define ARGUSPRINTPERCENTLOSS		33
   { "ploss", "", 8 , 1, ArgusPrintPercentLoss, ArgusPrintPercentLossLabel},
#define ARGUSPRINTSRCPERCENTLOSS	34
   { "sploss", "", 10 , 1, ArgusPrintSrcPercentLoss, ArgusPrintSrcPercentLossLabel},
#define ARGUSPRINTDSTPERCENTLOSS	35
   { "dploss", "", 10 , 1, ArgusPrintDstPercentLoss, ArgusPrintDstPercentLossLabel},
#define ARGUSPRINTRATE			36
   { "rate", "", 12 , 1, ArgusPrintRate, ArgusPrintRateLabel},
#define ARGUSPRINTSRCRATE		37
   { "srate", "", 12 , 1, ArgusPrintSrcRate, ArgusPrintSrcRateLabel},
#define ARGUSPRINTDSTRATE		38
   { "drate", "", 12 , 1, ArgusPrintDstRate, ArgusPrintDstRateLabel},
#define ARGUSPRINTSOURCEID		39
   { "srcid", "", 18 , 1, ArgusPrintSourceID, ArgusPrintSourceIDLabel},
#define ARGUSPRINTFLAGS			40
   { "flgs", "", 7 , 1, ArgusPrintFlags, ArgusPrintFlagsLabel},
#define ARGUSPRINTSRCMACADDRESS		41
   { "smac", "", 18 , 1, ArgusPrintSrcMacAddress, ArgusPrintSrcMacAddressLabel},
#define ARGUSPRINTDSTMACADDRESS		42
   { "dmac", "", 18 , 1, ArgusPrintDstMacAddress, ArgusPrintDstMacAddressLabel},
#define ARGUSPRINTDIR			43
   { "dir", "", 5 , 1, ArgusPrintDir, ArgusPrintDirLabel},
#define ARGUSPRINTSRCINTPKT		44
   { "sintpkt", "", 12 , 1, ArgusPrintSrcIntPkt, ArgusPrintSrcIntPktLabel},
#define ARGUSPRINTDSTINTPKT		45
   { "dintpkt", "", 12 , 1, ArgusPrintDstIntPkt, ArgusPrintDstIntPktLabel},
#define ARGUSPRINTACTSRCINTPKT		46
   { "sintpktact", "", 12 , 1, ArgusPrintActiveSrcIntPkt, ArgusPrintActiveSrcIntPktLabel},
#define ARGUSPRINTACTDSTINTPKT		47
   { "dintpktact", "", 12 , 1, ArgusPrintActiveDstIntPkt, ArgusPrintActiveDstIntPktLabel},
#define ARGUSPRINTIDLESRCINTPKT		48
   { "sintpktidl", "", 12 , 1, ArgusPrintIdleSrcIntPkt, ArgusPrintIdleSrcIntPktLabel},
#define ARGUSPRINTIDLEDSTINTPKT		49
   { "dintpktidl", "", 12 , 1, ArgusPrintIdleDstIntPkt, ArgusPrintIdleDstIntPktLabel},
#define ARGUSPRINTSRCINTPKTMAX		50
   { "sintpktmax", "", 12 , 1, ArgusPrintSrcIntPktMax, ArgusPrintSrcIntPktMaxLabel},
#define ARGUSPRINTSRCINTPKTMIN		51
   { "sintpktmin", "", 12 , 1, ArgusPrintSrcIntPktMin, ArgusPrintSrcIntPktMinLabel},
#define ARGUSPRINTDSTINTPKTMAX		52
   { "dintpktmax", "", 12 , 1, ArgusPrintDstIntPktMax, ArgusPrintDstIntPktMaxLabel},
#define ARGUSPRINTDSTINTPKTMIN		53
   { "dintpktmin", "", 12 , 1, ArgusPrintDstIntPktMin, ArgusPrintDstIntPktMinLabel},
#define ARGUSPRINTACTSRCINTPKTMAX	54
   { "sintpktactmax", "", 12 , 1, ArgusPrintActiveSrcIntPktMax, ArgusPrintActiveSrcIntPktMaxLabel},
#define ARGUSPRINTACTSRCINTPKTMIN	55
   { "sintpktactmin", "", 12 , 1, ArgusPrintActiveSrcIntPktMin, ArgusPrintActiveSrcIntPktMinLabel},
#define ARGUSPRINTACTDSTINTPKTMAX	56
   { "dintpktactmax", "", 12 , 1, ArgusPrintActiveDstIntPktMax, ArgusPrintActiveDstIntPktMaxLabel},
#define ARGUSPRINTACTDSTINTPKTMIN	57
   { "dintpktactmin", "", 12 , 1, ArgusPrintActiveDstIntPktMin, ArgusPrintActiveDstIntPktMinLabel},
#define ARGUSPRINTIDLESRCINTPKTMAX	58
   { "sintpktidlmax", "", 12 , 1, ArgusPrintIdleSrcIntPktMax, ArgusPrintIdleSrcIntPktMaxLabel},
#define ARGUSPRINTIDLESRCINTPKTMIN	59
   { "sintpktidlmin", "", 12 , 1, ArgusPrintIdleSrcIntPktMin, ArgusPrintIdleSrcIntPktMinLabel},
#define ARGUSPRINTIDLEDSTINTPKTMAX	60
   { "dintpktidlmax", "", 12 , 1, ArgusPrintIdleDstIntPktMax, ArgusPrintIdleDstIntPktMaxLabel},
#define ARGUSPRINTIDLEDSTINTPKTMIN	61
   { "dintpktidlmin", "", 12 , 1, ArgusPrintIdleDstIntPktMin, ArgusPrintIdleDstIntPktMinLabel},
#define ARGUSPRINTSPACER		62
   { "xxx", "", 12 , 1, NULL, NULL},
#define ARGUSPRINTSRCJITTER		63
   { "sjit", "", 12 , 1, ArgusPrintSrcJitter, ArgusPrintSrcJitterLabel},
#define ARGUSPRINTDSTJITTER		64
   { "djit", "", 12 , 1, ArgusPrintDstJitter, ArgusPrintDstJitterLabel},
#define ARGUSPRINTACTSRCJITTER		65
   { "sjitact", "", 12 , 1, ArgusPrintActiveSrcJitter, ArgusPrintActiveSrcJitterLabel},
#define ARGUSPRINTACTDSTJITTER		66
   { "djitact", "", 12 , 1, ArgusPrintActiveDstJitter, ArgusPrintActiveDstJitterLabel},
#define ARGUSPRINTIDLESRCJITTER		67
   { "sjitidl", "", 12 , 1, ArgusPrintIdleSrcJitter, ArgusPrintIdleSrcJitterLabel},
#define ARGUSPRINTIDLEDSTJITTER		68
   { "djitidl", "", 12 , 1, ArgusPrintIdleDstJitter, ArgusPrintIdleDstJitterLabel},
#define ARGUSPRINTSTATE			69
   { "state", "", 5 , 1, ArgusPrintState, ArgusPrintStateLabel},
#define ARGUSPRINTDELTADURATION		70
   { "dldur", "", 12 , 1, ArgusPrintDeltaDuration, ArgusPrintDeltaDurationLabel},
#define ARGUSPRINTDELTASTARTTIME	71
   { "delstime", "", 12 , 1, ArgusPrintDeltaStartTime, ArgusPrintDeltaStartTimeLabel},
#define ARGUSPRINTDELTALASTTIME		72
   { "delltime", "", 12 , 1, ArgusPrintDeltaLastTime, ArgusPrintDeltaLastTimeLabel},
#define ARGUSPRINTDELTASRCPKTS		73
   { "dspkts", "", 12 , 1, ArgusPrintDeltaSrcPkts, ArgusPrintDeltaSrcPktsLabel},
#define ARGUSPRINTDELTADSTPKTS		74
   { "ddpkts", "", 12 , 1, ArgusPrintDeltaDstPkts, ArgusPrintDeltaDstPktsLabel},
#define ARGUSPRINTDELTASRCBYTES		75
   { "dsbytes", "", 12 , 1, ArgusPrintDeltaSrcBytes, ArgusPrintDeltaSrcBytesLabel},
#define ARGUSPRINTDELTADSTBYTES		76
   { "ddbytes", "", 12 , 1, ArgusPrintDeltaDstBytes, ArgusPrintDeltaDstBytesLabel},
#define ARGUSPRINTPERCENTDELTASRCPKTS	77
   { "pdspkts", "", 12 , 1, ArgusPrintPercentDeltaSrcPkts, ArgusPrintPercentDeltaSrcPktsLabel},
#define ARGUSPRINTPERCENTDELTADSTPKTS	78
   { "pddpkts", "", 12 , 1, ArgusPrintPercentDeltaDstPkts, ArgusPrintPercentDeltaDstPktsLabel},
#define ARGUSPRINTPERCENTDELTASRCBYTES	79
   { "pdsbytes", "", 12 , 1, ArgusPrintPercentDeltaSrcBytes, ArgusPrintPercentDeltaSrcBytesLabel},
#define ARGUSPRINTPERCENTDELTADSTBYTES	80
   { "pddbytes", "", 12 , 1, ArgusPrintPercentDeltaDstBytes, ArgusPrintPercentDeltaDstBytesLabel},
#define ARGUSPRINTSRCUSERDATA		81
   { "suser", "", 16 , 1, ArgusPrintSrcUserData, ArgusPrintSrcUserDataLabel},
#define ARGUSPRINTDSTUSERDATA		82
   { "duser", "", 16 , 1, ArgusPrintDstUserData, ArgusPrintDstUserDataLabel},
#define ARGUSPRINTTCPEXTENSIONS		83
   { "tcpext", "", 12 , 1, ArgusPrintTCPExtensions, ArgusPrintTCPExtensionsLabel},
#define ARGUSPRINTSRCWINDOW		84
   { "swin", "", 6 , 1, ArgusPrintSrcWindow, ArgusPrintSrcWindowLabel},
#define ARGUSPRINTDSTWINDOW		85
   { "dwin", "", 6 , 1, ArgusPrintDstWindow, ArgusPrintDstWindowLabel},
#define ARGUSPRINTJOINDELAY		86
   { "jdelay", "", 12 , 1, ArgusPrintJoinDelay, ArgusPrintJoinDelayLabel},
#define ARGUSPRINTLEAVEDELAY		87
   { "ldelay", "", 12 , 1, ArgusPrintLeaveDelay, ArgusPrintLeaveDelayLabel},
#define ARGUSPRINTSEQUENCENUMBER	88
   { "seq", "", 12 , 1, ArgusPrintSequenceNumber, ArgusPrintSequenceNumberLabel},
#define ARGUSPRINTBINS			89
   { "bins", "", 6 , 1, ArgusPrintBins, ArgusPrintBinsLabel},
#define ARGUSPRINTBINNUMBER		90
   { "binnum", "", 6 , 1, ArgusPrintBinNumber, ArgusPrintBinNumberLabel},
#define ARGUSPRINTSRCMPLS		91
   { "smpls", "", 8 , 1, ArgusPrintSrcMpls, ArgusPrintSrcMplsLabel},
#define ARGUSPRINTDSTMPLS		92
   { "dmpls", "", 8 , 1, ArgusPrintDstMpls, ArgusPrintDstMplsLabel},
#define ARGUSPRINTSRCVLAN		93
   { "svlan", "", 8 , 1, ArgusPrintSrcVlan, ArgusPrintSrcVlanLabel},
#define ARGUSPRINTDSTVLAN		94
   { "dvlan", "", 8 , 1, ArgusPrintDstVlan, ArgusPrintDstVlanLabel},
#define ARGUSPRINTSRCVID		95
   { "svid", "", 6 , 1, ArgusPrintSrcVID, ArgusPrintSrcVIDLabel},
#define ARGUSPRINTDSTVID		96
   { "dvid", "", 6 , 1, ArgusPrintDstVID, ArgusPrintDstVIDLabel},
#define ARGUSPRINTSRCVPRI		97
   { "svpri", "", 6 , 1, ArgusPrintSrcVPRI, ArgusPrintSrcVPRILabel},
#define ARGUSPRINTDSTVPRI		98
   { "dvpri", "", 6 , 1, ArgusPrintDstVPRI, ArgusPrintDstVPRILabel},
#define ARGUSPRINTSRCIPID		99
   { "sipid", "", 7 , 1, ArgusPrintSrcIpId, ArgusPrintSrcIpIdLabel},
#define ARGUSPRINTDSTIPID		100
   { "dipid", "", 7 , 1, ArgusPrintDstIpId, ArgusPrintDstIpIdLabel},
#define ARGUSPRINTSRCRANGE		101
   { "srng", "", 6 , 1, ArgusPrintSrcRange, ArgusPrintSrcRangeLabel},
#define ARGUSPRINTDSTRANGE		102
   { "drng", "", 6 , 1, ArgusPrintDstRange, ArgusPrintDstRangeLabel},
#define ARGUSPRINTTCPSRCBASE		103
   { "stcpb", "", 12 , 1, ArgusPrintTCPSrcBase, ArgusPrintTCPSrcBaseLabel},
#define ARGUSPRINTTCPDSTBASE		104
   { "dtcpb", "", 12 , 1, ArgusPrintTCPDstBase, ArgusPrintTCPDstBaseLabel},
#define ARGUSPRINTTCPRTT		105
   { "tcprtt", "", 12 , 1, ArgusPrintTCPRTT, ArgusPrintTCPRTTLabel},
#define ARGUSPRINTINODE   		106
   { "inode", "", 18, 1, ArgusPrintInode, ArgusPrintInodeLabel},
#define ARGUSPRINTSTDDEV  		107
   { "stddev", "", 10 , 1, ArgusPrintStdDeviation, ArgusPrintStdDeviationLabel},
#define ARGUSPRINTRELDATE		108
   { "rtime", "", 12 , 1, ArgusPrintRelativeDate, ArgusPrintRelativeDateLabel},
#define ARGUSPRINTBYTEOFFSET		109
   { "offset", "", 12 , 1, ArgusPrintByteOffset, ArgusPrintByteOffsetLabel},
#define ARGUSPRINTSRCNET		110
   { "snet", "", 18 , 1, ArgusPrintSrcNet, ArgusPrintSrcNetLabel},
#define ARGUSPRINTDSTNET		111
   { "dnet", "", 18 , 1, ArgusPrintDstNet, ArgusPrintDstNetLabel},
#define ARGUSPRINTSRCDURATION		112
   { "sdur", "", 10 , 1, ArgusPrintSrcDuration, ArgusPrintSrcDurationLabel},
#define ARGUSPRINTDSTDURATION		113
   { "ddur", "", 10 , 1, ArgusPrintDstDuration, ArgusPrintDstDurationLabel},
#define ARGUSPRINTTCPSRCMAX		114
   { "stcpmax", "", 10 , 1, ArgusPrintTCPSrcMax, ArgusPrintTCPSrcMaxLabel},
#define ARGUSPRINTTCPDSTMAX		115
   { "dtcpmax", "", 10 , 1, ArgusPrintTCPDstMax, ArgusPrintTCPDstMaxLabel},
#define ARGUSPRINTTCPSYNACK		116
   { "synack", "", 12 , 1, ArgusPrintTCPSynAck, ArgusPrintTCPSynAckLabel},
#define ARGUSPRINTTCPACKDAT		117
   { "ackdat", "", 12 , 1, ArgusPrintTCPAckDat, ArgusPrintTCPAckDatLabel},
#define ARGUSPRINTSRCSTARTDATE		118
   { "sstime", "%T.%f", 12 , 1, ArgusPrintSrcStartDate, ArgusPrintSrcStartDateLabel},
#define ARGUSPRINTSRCLASTDATE		119
   { "sltime", "%T.%f", 12 , 1, ArgusPrintSrcLastDate, ArgusPrintSrcLastDateLabel},
#define ARGUSPRINTDSTSTARTDATE		120
   { "dstime", "%T.%f", 12 , 1, ArgusPrintDstStartDate, ArgusPrintDstStartDateLabel},
#define ARGUSPRINTDSTLASTDATE		121
   { "dltime", "%T.%f", 12 , 1, ArgusPrintDstLastDate, ArgusPrintDstLastDateLabel},
};


#define IPPROTOSTR              256

char *ip_proto_string [IPPROTOSTR] = {"ip", "icmp", "igmp", "ggp",
   "ipnip", "st2", "tcp", "cbt", "egp", "igp", "bbn-rcc", "nvp",
   "pup", "argus", "emcon", "xnet", "chaos", "udp", "mux", "dcn",
   "hmp", "prm", "xns-idp", "trunk-1", "trunk-2", "leaf-1", "leaf-2",
   "rdp", "irtp", "iso-tp4", "netblt", "mfe-nsp", "merit-inp", "sep",
   "3pc", "idpr", "xtp", "ddp", "idpr-cmtp", "tp++", "il", "ipv6",
   "sdrp", "ipv6-route", "ipv6-frag", "idrp", "rsvp", "gre", "mhrp", "bna",
   "esp", "ah", "i-nlsp", "swipe", "narp", "mobile", "tlsp", "skip",
   "ipv6-icmp", "ipv6-no", "ipv6-opts", "any", "cftp", "any", "sat-expak", "kryptolan",
   "rvd", "ippc", "any", "sat-mon", "visa", "ipcv", "cpnx", "cphb", "wsn",
   "pvp", "br-sat-mon", "sun-nd", "wb-mon", "wb-expak", "iso-ip", "vmtp",
   "secure-vmtp", "vines", "ttp", "nsfnet-igp", "dgp", "tcf", "eigrp",
   "ospf", "sprite-rpc", "larp", "mtp", "ax.25", "ipip", "micp",
   "aes-sp3-d", "etherip", "encap", "pri-enc", "gmtp", "ifmp", "pnni",
   "pim", "aris", "scps", "qnx", "a/n", "ipcomp", "snp", "compaq-peer",
   "ipx-n-ip", "vrrp", "pgm", "zero", "l2tp", "ddx", "iatp", "stp", "srp",
   "uti", "smp", "sm", "ptp", "isis", "fire", "crtp", "crudp", "sccopmce", "iplt",
   "sps", "pipe", "sctp", "fc", "rsvp", NULL, NULL, NULL, NULL, NULL, NULL,
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /*141-150*/
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /*151-160*/
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /*161-170*/
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /*171-180*/
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /*181-190*/
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /*191-200*/
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /*201-210*/
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /*211-220*/
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /*221-230*/
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /*231-240*/
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /*241-250*/
   NULL, NULL, "ib", NULL, NULL,                                /*251-255*/
};

char *icmptypestr[ICMP_MAXTYPE + 1] = {
   "ECR", "   ", "   ", "UR" , "SRC", "RED",
   "AHA", "   ", "ECO", "RTA", "RTS", "TXD",
   "PAR", "TST", "TSR", "IRQ", "IRR", "MAS",
   "MSR", "SEC", "ROB", "ROB", "ROB", "ROB",
   "ROB", "ROB", "ROB", "ROB", "ROB", "ROB",
   "TRC", "DCE", "MHR", "WAY", "IAH", "MRQ",
   "MRP", "DNQ", "DNP", "SKP", "PHO", "NDS",
   "NDA", "NDN", "NDR", "PTB",
};   

struct ArgusTokenStruct llcsap_db[] = {
   { LLCSAP_NULL,   "null" },
   { LLCSAP_8021B_I,   "gsap" },
   { LLCSAP_8021B_G,   "isap" },
   { LLCSAP_SNAPATH,   "snapath" },
   { LLCSAP_IP,      "ipsap" },
   { LLCSAP_SNA1,   "sna1" },
   { LLCSAP_SNA2,   "sna2" },
   { LLCSAP_PROWAYNM,   "p-nm" },
   { LLCSAP_TI,      "ti" },
   { LLCSAP_BPDU,   "stp" },
   { LLCSAP_RS511,   "eia" },
   { LLCSAP_ISO8208,   "x25" },
   { LLCSAP_XNS,   "xns" },
   { LLCSAP_NESTAR,   "nestar" },
   { LLCSAP_PROWAYASLM,   "p-aslm" },
   { LLCSAP_ARP,   "arp" },
   { LLCSAP_SNAP,   "snap" },
   { LLCSAP_VINES1,   "vine1" },
   { LLCSAP_VINES2,   "vine2" },
   { LLCSAP_NETWARE,   "netware" },
   { LLCSAP_NETBIOS,   "netbios" },
   { LLCSAP_IBMNM,   "ibmnm" },
   { LLCSAP_RPL1,   "rpl1" },
   { LLCSAP_UB,      "ub" },
   { LLCSAP_RPL2,   "rpl2" },
   { LLCSAP_ISONS,   "clns" },
   { LLCSAP_GLOBAL,   "gbl" },
   { 0,             NULL }
};

void ArgusInitServarray(struct ArgusParserStruct *);
void ArgusInitEprotoarray(void);
void ArgusInitProtoidarray(void);
void ArgusInitEtherarray(void);
void ArgusInitLlcsaparray(void);
void ArgusInitAddrtoname(struct ArgusParserStruct *, u_int, u_int);

unsigned int ArgusIndexRecord (struct ArgusRecordStruct *);

void ArgusFree (void *buf);
void *ArgusMalloc (int);
void *ArgusCalloc (int, int);
void *ArgusMallocListRecord (int length);
void ArgusFreeListRecord (void *buf);
int ArgusParseResourceFile (struct ArgusParserStruct *, char *);
int ArgusParseTimeArg (char **, char **, int, struct tm *);

void ArgusAdjustGlobalTime (struct ArgusParserStruct *parser, struct timeval *now);
void ArgusReverseRecordWithFlag (struct ArgusRecordStruct *, int); 
void ArgusReverseRecord (struct ArgusRecordStruct *); 
void ArgusReverseDataRecord (struct ArgusRecordStruct *); 
void ArgusZeroRecord (struct ArgusRecordStruct *); 
void ArgusZeroRecordWithFlag (struct ArgusRecordStruct *, int); 
struct ArgusRecordStruct *ArgusSubtractRecord (struct ArgusRecordStruct *, struct ArgusRecordStruct *);

struct ArgusQueueStruct *ArgusNewQueue (void);
void ArgusDeleteQueue (struct ArgusQueueStruct *);
int ArgusGetQueueCount(struct ArgusQueueStruct *);
void ArgusPushQueue(struct ArgusQueueStruct *, struct ArgusQueueHeader *);
struct ArgusQueueHeader *ArgusPopQueue (struct ArgusQueueStruct *queue);
int ArgusAddToQueue(struct ArgusQueueStruct *, struct ArgusQueueHeader *);
struct ArgusQueueHeader *ArgusRemoveFromQueue(struct ArgusQueueStruct *, struct ArgusQueueHeader *);

int ArgusConvertInitialWriteStruct (struct WriteStruct *, struct ArgusRecordStruct *);
int ArgusConvertWriteStruct (struct WriteStruct *, struct ArgusRecordStruct *);

struct timeval *RaMinTime (struct timeval *, struct timeval *);
struct timeval *RaMaxTime (struct timeval *, struct timeval *);
struct timeval *RaDiffTime (struct timeval *, struct timeval *);

void ArgusPrintTime(struct ArgusParserStruct *, char *, struct timeval *);
char *ArgusGenerateLabel(struct ArgusParserStruct *, struct ArgusRecordStruct *);

void ArgusPrintRecord (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *ptr, int);
void ArgusDump (const u_char *, int, char *);


char *RaGetUserDataString (struct ArgusRecordStruct *);

int ArgusEncode (struct ArgusParserStruct *, const char *, const char *, int, char *, int);
int ArgusEncode32 (struct ArgusParserStruct *, const char *, int , char *, int );

int ArgusEncode64 (struct ArgusParserStruct *, const char *, int, char *, int);
int ArgusEncodeAscii (struct ArgusParserStruct *, const char *, int, char *, int);

void clearArgusWfile(struct ArgusParserStruct *);
extern unsigned int thisnet, localaddr, localnet, netmask;

void (*RaPrintAlgorithms[ARGUS_MAX_PRINT_ALG])(struct ArgusParserStruct *, char *, struct ArgusRecordStruct *) = {
   ArgusPrintStartDate,
   ArgusPrintFlags,
   ArgusPrintProto,
   ArgusPrintSrcAddr,
   ArgusPrintSrcPort,
   ArgusPrintDir,
   ArgusPrintDstAddr,
   ArgusPrintDstPort,
   ArgusPrintSrcPackets,
   ArgusPrintDstPackets,
   ArgusPrintSrcBytes,
   ArgusPrintDstBytes,
   ArgusPrintState,
   NULL,
};



int argus_nametoeproto(char *);
unsigned int __argus_atoin(char *, unsigned int *);


void ArgusNtoH (struct ArgusRecord *);
void ArgusHtoN (struct ArgusRecord *);

void ArgusV2NtoH (struct ArgusV2Record *);
void ArgusV2HtoN (struct ArgusV2Record *);

#else
#define ARGUSPRINTSTARTDATE		0
#define ARGUSPRINTLASTDATE		1
#define ARGUSPRINTTRANSACTIONS		2
#define ARGUSPRINTDURATION		3
#define ARGUSPRINTAVGDURATION		4
#define ARGUSPRINTMINDURATION		5
#define ARGUSPRINTMAXDURATION		6
#define ARGUSPRINTSRCADDR		7
#define ARGUSPRINTDSTADDR		8
#define ARGUSPRINTPROTO			9
#define ARGUSPRINTSRCPORT		10
#define ARGUSPRINTDSTPORT		11
#define ARGUSPRINTSRCTOS		12
#define ARGUSPRINTDSTTOS		13
#define ARGUSPRINTSRCDSBYTE		14
#define ARGUSPRINTDSTDSBYTE		15
#define ARGUSPRINTSRCTTL		16
#define ARGUSPRINTDSTTTL		17
#define ARGUSPRINTBYTES			18
#define ARGUSPRINTSRCBYTES		19
#define ARGUSPRINTDSTBYTES		20
#define ARGUSPRINTAPPBYTES              21
#define ARGUSPRINTSRCAPPBYTES           22
#define ARGUSPRINTDSTAPPBYTES           23
#define ARGUSPRINTPACKETS		24
#define ARGUSPRINTSRCPACKETS		25
#define ARGUSPRINTDSTPACKETS		26
#define ARGUSPRINTLOAD			27
#define ARGUSPRINTSRCLOAD		28
#define ARGUSPRINTDSTLOAD		29
#define ARGUSPRINTLOSS			30
#define ARGUSPRINTSRCLOSS		31
#define ARGUSPRINTDSTLOSS		32
#define ARGUSPRINTPERCENTLOSS		33
#define ARGUSPRINTSRCPERCENTLOSS	34
#define ARGUSPRINTDSTPERCENTLOSS	35
#define ARGUSPRINTRATE			36
#define ARGUSPRINTSRCRATE		37
#define ARGUSPRINTDSTRATE		38
#define ARGUSPRINTSOURCEID		39
#define ARGUSPRINTFLAGS			40
#define ARGUSPRINTSRCMACADDRESS		41
#define ARGUSPRINTDSTMACADDRESS		42
#define ARGUSPRINTDIR			43
#define ARGUSPRINTSRCINTPKT		44
#define ARGUSPRINTDSTINTPKT		45
#define ARGUSPRINTACTSRCINTPKT		46
#define ARGUSPRINTACTDSTINTPKT		47
#define ARGUSPRINTIDLESRCINTPKT		48
#define ARGUSPRINTIDLEDSTINTPKT		49
#define ARGUSPRINTSRCINTPKTMAX		50
#define ARGUSPRINTSRCINTPKTMIN		51
#define ARGUSPRINTDSTINTPKTMAX		52
#define ARGUSPRINTDSTINTPKTMIN		53
#define ARGUSPRINTACTSRCINTPKTMAX	54
#define ARGUSPRINTACTSRCINTPKTMIN	55
#define ARGUSPRINTACTDSTINTPKTMAX	56
#define ARGUSPRINTACTDSTINTPKTMIN	57
#define ARGUSPRINTIDLESRCINTPKTMAX	58
#define ARGUSPRINTIDLESRCINTPKTMIN	59
#define ARGUSPRINTIDLEDSTINTPKTMAX	60
#define ARGUSPRINTIDLEDSTINTPKTMIN	61
#define ARGUSPRINTSRCJITTER		63
#define ARGUSPRINTDSTJITTER		64
#define ARGUSPRINTACTSRCJITTER		65
#define ARGUSPRINTACTDSTJITTER		66
#define ARGUSPRINTIDLESRCJITTER		67
#define ARGUSPRINTIDLEDSTJITTER		68
#define ARGUSPRINTSTATE			69
#define ARGUSPRINTDELTADURATION		70
#define ARGUSPRINTDELTASTARTTIME	71
#define ARGUSPRINTDELTALASTTIME		72
#define ARGUSPRINTDELTASRCPKTS		73
#define ARGUSPRINTDELTADSTPKTS		74
#define ARGUSPRINTDELTASRCBYTES		75
#define ARGUSPRINTDELTADSTBYTES		76
#define ARGUSPRINTPERCENTDELTASRCPKTS	77
#define ARGUSPRINTPERCENTDELTADSTPKTS	78
#define ARGUSPRINTPERCENTDELTASRCBYTES	79
#define ARGUSPRINTPERCENTDELTADSTBYTES	80
#define ARGUSPRINTSRCUSERDATA		81
#define ARGUSPRINTDSTUSERDATA		82
#define ARGUSPRINTTCPEXTENSIONS		83
#define ARGUSPRINTSRCWINDOW		84
#define ARGUSPRINTDSTWINDOW		85
#define ARGUSPRINTJOINDELAY		86
#define ARGUSPRINTLEAVEDELAY		87
#define ARGUSPRINTSEQUENCENUMBER	88
#define ARGUSPRINTBINS			89
#define ARGUSPRINTBINNUMBER		90
#define ARGUSPRINTSRCMPLS		91
#define ARGUSPRINTDSTMPLS		92
#define ARGUSPRINTSRCVLAN		93
#define ARGUSPRINTDSTVLAN		94
#define ARGUSPRINTSRCVID		95
#define ARGUSPRINTDSTVID		96
#define ARGUSPRINTSRCVPRI		97
#define ARGUSPRINTDSTVPRI		98
#define ARGUSPRINTSRCIPID		99
#define ARGUSPRINTDSTIPID		100
#define ARGUSPRINTSRCRANGE		101
#define ARGUSPRINTDSTRANGE		102
#define ARGUSPRINTTCPSRCBASE		103
#define ARGUSPRINTTCPDSTBASE		104
#define ARGUSPRINTTCPRTT		105
#define ARGUSPRINTINODE   		106
#define ARGUSPRINTSTDDEV  		107
#define ARGUSPRINTRELDATE		108
#define ARGUSPRINTBYTEOFFSET		109
#define ARGUSPRINTSRCNET		110
#define ARGUSPRINTDSTNET		111
#define ARGUSPRINTSRCDURATION		112
#define ARGUSPRINTDSTDURATION		113
#define ARGUSPRINTTCPSRCMAX		114
#define ARGUSPRINTTCPDSTMAX		115
#define ARGUSPRINTTCPSYNACK		116
#define ARGUSPRINTTCPACKDAT		117

extern struct ArgusPrintFieldStruct RaPrintAlgorithmTable[MAX_PRINT_ALG_TYPES];
extern void (*RaPrintAlgorithms[ARGUS_MAX_PRINT_ALG])(struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);

extern void ArgusInitServarray(void);
extern void ArgusInitEprotoarray(void);
extern void ArgusInitProtoidarray(void);
extern void ArgusInitEtherarray(void);
extern void ArgusInitLlcsaparray(void);
extern void ArgusInitAddrtoname(struct ArgusParserStruct *, u_int, u_int);

extern char *ip_proto_string [];
extern char *icmptypestr[];

extern unsigned int ArgusIndexRecord (struct ArgusRecordStruct *);

extern void ArgusFree (void *buf);
extern void *ArgusMalloc (int);
extern void *ArgusCalloc (int, int);
extern void *ArgusMallocListRecord (int length);
extern void ArgusFreeListRecord (void *buf);
extern int ArgusParseResourceFile (struct ArgusParserStruct *, char *);

extern int ArgusParseTimeArg (char **, char **, int, struct tm *);

extern void ArgusAdjustGlobalTime (struct ArgusParserStruct *parser, struct timeval *now);
extern void ArgusReverseRecordWithFlag (struct ArgusRecordStruct *, int);
extern void ArgusReverseRecord (struct ArgusRecordStruct *); 
extern void ArgusReverseDataRecord (struct ArgusRecordStruct *); 
extern void ArgusZeroRecord (struct ArgusRecordStruct *);  
extern void ArgusZeroRecordWithFlag (struct ArgusRecordStruct *, int); 
extern struct ArgusRecordStruct *ArgusSubtractRecord (struct ArgusRecordStruct *, struct ArgusRecordStruct *);

extern struct ArgusQueueStruct *ArgusNewQueue (void);
extern void ArgusDeleteQueue (struct ArgusQueueStruct *);
extern int ArgusGetQueueCount(struct ArgusQueueStruct *);
extern void ArgusPushQueue(struct ArgusQueueStruct *, struct ArgusQueueHeader *);
extern struct ArgusQueueHeader *ArgusPopQueue (struct ArgusQueueStruct *queue);
extern int ArgusAddToQueue(struct ArgusQueueStruct *, struct ArgusQueueHeader *);
extern struct ArgusQueueHeader *ArgusRemoveFromQueue(struct ArgusQueueStruct *, struct ArgusQueueHeader *);

extern int ArgusConvertInitialWriteStruct (struct WriteStruct *, struct ArgusRecordStruct *);
extern int ArgusConvertWriteStruct (struct WriteStruct *, struct ArgusRecordStruct *);

extern struct timeval *RaMinTime (struct timeval *, struct timeval *);
extern struct timeval *RaMaxTime (struct timeval *, struct timeval *);
extern struct timeval *RaDiffTime (struct timeval *, struct timeval *);

extern void ArgusPrintTime(struct ArgusParserStruct *, char *, struct timeval *);
extern char *ArgusGenerateLabel(struct ArgusParserStruct *, struct ArgusRecordStruct *);

extern void ArgusPrintRecord (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *ptr, int);
extern void ArgusDump (const u_char *, int, char *);

extern void ArgusPrintDate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintStartDate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintLastDate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcStartDate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcLastDate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstStartDate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstLastDate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSourceID (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintFlags (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintMacAddress (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcMacAddress (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstMacAddress (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintProto (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintAddr (struct ArgusParserStruct *, char *, int, void *, int, int, int);
extern void ArgusPrintSrcNet (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcAddr (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstNet (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstAddr (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintPort (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *, int, unsigned char, unsigned short, int, int);
extern void ArgusPrintSrcPort (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstPort (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDir (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintPackets (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcPackets (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstPackets (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintAppBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcAppBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstAppBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintIntPkt (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcIntPkt (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstIntPkt (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcIntPktMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcIntPktMin (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstIntPktMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstIntPktMin (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintActiveSrcIntPktMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintActiveSrcIntPktMin (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintActiveDstIntPktMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintActiveDstIntPktMin (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintIdleSrcIntPktMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintIdleSrcIntPktMin (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintIdleDstIntPktMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintIdleDstIntPktMin (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintJitter (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcJitter (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstJitter (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintState (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDeltaDuration (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDeltaStartTime (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDeltaLastTime (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDeltaSrcPkts (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDeltaDstPkts (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDeltaSrcBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDeltaDstBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintPercentDeltaSrcPkts (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintPercentDeltaDstPkts (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintPercentDeltaSrcBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintPercentDeltaDstBytes (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcUserData (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstUserData (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);

extern void ArgusPrintTCPExtensions (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcLoad (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstLoad (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintLoad (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcLoss (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstLoss (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintLoss (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintPercentLoss (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcRate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstRate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintRate (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);

extern void ArgusPrintSrcTos (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstTos (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);

extern void ArgusPrintSrcDSByte (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstDSByte (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcIpId (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstIpId (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);

extern void ArgusPrintSrcTtl (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstTtl (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);

extern void ArgusPrintSrcVlan (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstVlan (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);

extern void ArgusPrintSrcVID (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstVID (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);

extern void ArgusPrintSrcVPRI (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstVPRI (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);

extern void ArgusPrintSrcMpls (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstMpls (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);


extern void ArgusPrintSrcWindow (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstWindow (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintJoinDelay (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintLeaveDelay (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintAvgDuration (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcRange (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstRange (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcDuration (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstDuration (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDuration (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintTransactions (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSequenceNumber (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintBinNumber (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintBins (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintTCPSrcBase (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintTCPDstBase (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintTCPRTT (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintTCPSrcMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintTCPDstMax (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintInode (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintMinDuration (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintMaxDuration (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintStdDeviation (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);

extern void ArgusPrintDateLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintStartDateLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintLastDateLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSourceIDLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintFlagsLabel (struct ArgusParserStruct *, char *);

extern void ArgusPrintSrcMacAddressLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstMacAddressLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintProtoLabel (struct ArgusParserStruct *, char *);


extern void ArgusPrintSrcAddrLabel (struct ArgusParserStruct *, char *);

extern void ArgusPrintDstAddrLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcPortLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstPortLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcIpIdLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstIpIdLabel (struct ArgusParserStruct *, char *);

extern void ArgusPrintSrcTtlLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstTtlLabel (struct ArgusParserStruct *, char *);

extern void ArgusPrintDirLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintPacketsLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcPacketsLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstPacketsLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintBytesLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcBytesLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstBytesLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintAppBytesLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcAppBytesLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstAppBytesLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintIntPktLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcIntPktLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstIntPktLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcIntPktMaxLabel (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintSrcIntPktMinLabel (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstIntPktMaxLabel (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintDstIntPktMinLabel (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintActiveSrcIntPktMaxLabel (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintActiveSrcIntPktMinLabel (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintActiveDstIntPktMaxLabel (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintActiveDstIntPktMinLabel (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintIdleSrcIntPktMaxLabel (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintIdleSrcIntPktMinLabel (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintIdleDstIntPktMaxLabel (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintIdleDstIntPktMinLabel (struct ArgusParserStruct *, char *, struct ArgusRecordStruct *);
extern void ArgusPrintJitterLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcJitterLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstJitterLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintStateLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDeltaDurationLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDeltaStartTimeLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDeltaLastTimeLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDeltaSrcPktsLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDeltaDstPktsLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDeltaSrcBytesLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDeltaDstBytesLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintPercentDeltaSrcPktsLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintPercentDeltaDstPktsLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintPercentDeltaSrcBytesLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintPercentDeltaDstBytesLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcUserDataLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstUserDataLabel (struct ArgusParserStruct *, char *);

extern void ArgusPrintTCPExtensionsLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcLoadLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstLoadLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcLoadLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstLoadLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintLoadLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcLossLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstLossLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintLossLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintPercentLossLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcRateLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstRateLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintRateLabel (struct ArgusParserStruct *, char *);

extern void ArgusPrintSrcTosLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstTosLabel (struct ArgusParserStruct *, char *);

extern void ArgusPrintSrcDSByteLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstDSByteLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcVlanLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstVlanLabel (struct ArgusParserStruct *, char *);

extern void ArgusPrintSrcVIDLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstVIDLabel (struct ArgusParserStruct *, char *);

extern void ArgusPrintSrcVPRILabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstVPRILabel (struct ArgusParserStruct *, char *);

extern void ArgusPrintSrcMplsLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstMplsLabel (struct ArgusParserStruct *, char *);

extern void ArgusPrintWindowLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcWindowLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstWindowLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintJoinDelayLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintLeaveDelayLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintAvgDurationLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcRangeLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDstRangeLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcDurationLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSrcDurationLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintDurationLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintTransactionsLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintSequenceNumberLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintBinNumberLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintBinsLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintTCPSrcBaseLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintTCPDstBaseLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintTCPRTTLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintTCPSrcMaxLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintTCPDstMaxLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintInodeLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintMinDurationLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintMaxDurationLabel (struct ArgusParserStruct *, char *);
extern void ArgusPrintStdDeviationLabel (struct ArgusParserStruct *, char *);

extern char *RaGetUserDataString (struct ArgusRecordStruct *);

extern int ArgusEncode (struct ArgusParserStruct *, const char *, const char *, int, char *, int);
extern int ArgusEncode32 (struct ArgusParserStruct *, const char *, int , char *, int );
extern int ArgusEncode64 (const char *, int, char *, int);
extern int ArgusEncodeAscii (const char *, int, char *, int);

extern int argus_nametoeproto(char *);
extern unsigned int __argus_atoin(char *, unsigned int *);


extern void ArgusNtoH (struct ArgusRecord *);
extern void ArgusHtoN (struct ArgusRecord *);

extern void ArgusV2NtoH (struct ArgusV2Record *);
extern void ArgusV2HtoN (struct ArgusV2Record *);

#endif 
#endif /* ArgusUtil_h */
